const { getIpAddress } = require('./network-utils');
const chalk = require('chalk');
const Koa = require('koa');
const Router = require('koa-router');
const router = new Router();
const app = new Koa();
const BodyParser = require('koa-bodyparser');
const bodyparser = new BodyParser();
const axios = require('axios');
const crypto = require('crypto');
const { v4: uuidv4 } = require('uuid');
// 存储token
const tokens = {};

/**
 * @description: 解密方法
 * @param {string} encryptedData
 * @param {string} sessionKey
 * @param {string} iv
 * @return {string}
 */
function decrypt(encryptedData, sessionKey, iv) {
  const decipher = crypto.createDecipheriv('aes-128-cbc', Buffer.from(sessionKey, 'base64'), Buffer.from(iv, 'base64'));
  let ret = decipher.update(encryptedData, 'base64', 'utf8');
  ret += decipher.final('utf8');
  return ret;
}

/**
 * @description: 字符串脱敏，中间四个字符被和谐
 * @param {*} str
 * @return {*}
 */
function desensitization(str) {
  const len = str.length;
  const mid = len >> 1;
  return `${str.substring(0, mid - 1)}****${str.substring(mid + 2)}`;
}

/**
 * @description: 启动服务
 * @param {string} secret 小程序secret
 * @param {string} appid 小程序appid
 * @param {boolean} sandbox 是否为沙盒环境
 * @param {number} port 本地服务启动的端口号
 * @return {void}
 */
function createServer(secret, appid, sandbox, port) {
  // 根据是否启用沙盒环境确定请求抖音开平的URL
  const baseURL = sandbox ? 'https://developer.toutiao.com' : 'https://open-sandbox.douyin.com';
  const http = axios.create({
    baseURL,
  });
  // 返回用户政策或隐私协议的代码示例
  router.get('/', ctx => {
    ctx.set('content-type', 'text/html');
    ctx.body = `<!DOCTYPE html>
    <html lang="en">
    <head>
        <meta charset="UTF-8">
        <meta http-equiv="X-UA-Compatible" content="IE=edge">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
        <title>用户协议</title>
    </head>
    <body>
        <h1>
            用户协议
        </h1>
        <p>xxxxx</p>
        <p>xxxxxxxxxxxxxxxxxxx</p>
    </body>
    </html>`;
  });

  // 静默登录接口
  router.post('/api/apps/login', async ctx => {
    // 前端上报tt.login返回的code
    const { code } = ctx.request.body;
    // 调用开平code2session接口
    const res = await http.request({
      url: '/api/apps/v2/jscode2session',
      method: 'post',
      data: {
        secret,
        code,
        appid,
      },
    });
    // 获取到session_key,用户openid和union_id
    const { session_key, openid, unionid } = res.data.data;
    // 根据openid生成token
    const token = uuidv4();
    // 存储到开发者服务端
    tokens[token] = {
      session_key,
      openid,
      unionid,
      token,
    };

    ctx.body = {
      errCode: 0,
      errMsg: 'ok',
      token,
      openid: desensitization(openid),
      unionid: desensitization(unionid),
    };
  });

  // 解密用户数据
  router.post('/api/apps/showData', ctx => {
    /**
     * @description: 前端需要上报tt.login返回的code和tt.getUserProfile和getPhoneNumber返回的encryptedData
     * @param {{ data: string, iv: string,dataType: number }[]} encryptedData: data(用户加密数据)，iv( 随机iv), dataType(1为获取电话；2为获取用户信息)
     * @return {*}
     */
    const { encryptedData, token } = ctx.request.body;

    let info = tokens[token];

    // 获取本地缓存sessionKey
    const { session_key } = info;

    const { data, iv } = encryptedData;
    // 解密
    let decryptedData = null;
    try {
      decryptedData = decrypt(data, session_key, iv);
      decryptedData = JSON.parse(decryptedData);
      tokens[token].userData = decryptedData;
      // 此处做了一个脱敏处理
      decryptedData.phoneNumber = desensitization(decryptedData.phoneNumber);
      decryptedData.purePhoneNumber = desensitization(decryptedData.purePhoneNumber);
      // 下发前端解密数据
      ctx.body = {
        errCode: 0,
        errMsg: 'ok',
        decryptedData: JSON.stringify(decryptedData),
      };
    } catch (err) {
      console.log(err);
      // 若解密失败，说明sessionKey过期，用户登录失效
      ctx.status = 401;
      ctx.body = {
        errCode: 1,
        errMsg: '认证过期',
      };
      delete tokens[token];
    }
  });

  // 确定用户登录后调用的返回用户信息接口，需要前端上传token
  router.post('/api/apps/userData', ctx => {
    const { token } = ctx.request.body;
    const user = tokens[token];
    if (!user) {
      ctx.status = 404;
      ctx.body = {
        errCode: 1,
        errMsg: '未找到用户',
        userData: {},
      };
      return;
    }
    const decryptedData = user && user.userData;
    if (!decryptedData) {
      ctx.status = 400;
      ctx.body = {
        errCode: 2,
        errMsg: '请先调用showData接口',
        userData: {},
      };
      return;
    }
    // 此处做了一个脱敏处理
    decryptedData.phoneNumber = desensitization(decryptedData.phoneNumber);
    decryptedData.purePhoneNumber = desensitization(decryptedData.purePhoneNumber);
    ctx.body = {
      errCode: 0,
      errMsg: 'ok',
      userData: { data: JSON.stringify(decryptedData) },
    };
  });

  router.post('/api/apps/logout', ctx => {
    const { token } = ctx.request.body;
    delete tokens[token];
    ctx.body = {
      errCode: 0,
      errMsg: 'logout successfully',
    };
  });
  app.use(bodyparser);
  app.use(router.routes());
  app.use(router.allowedMethods());

  app.listen(port, () => {
    console.log('\n', chalk.green('服务器启动成功!'));
    console.log(`\n\t${chalk.bold('本地地址：')}${chalk.cyan.underline(`http://127.0.0.1:${port}`)}`);
    console.log(`\n\t${chalk.bold('局域网地址：')}${chalk.cyan.underline(`http://${getIpAddress()}:${port}`)}`);
  });
}
module.exports = {
  createServer,
};
